/** 
 *   Camera Control for STAR I.
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdlib.h>
#include <math.h>
#include <stdio.h>


double mod24(double x)
{
	while (x < 0) x += 24.0;
	while (x > 24.0) x -= 24.0;
			 
	return x;
}


double strdeg2double(char *str)
{
	double g = 0.0,
		m = 0.0,
		s = 0.0;
	double sig = 1.0;
	char *s1, *s2;

	s1 = str;
	g = strtod(s1, &s2);
	if(g < 0) sig = -1.0;
	if( s2 != NULL ) {
		s1 = s2+1;
		m = strtod(s1, &s2);
		if(s2 != NULL){
			s1 = s2+1;
			s = strtod(s1, &s2);
		}
	}

	return g + (sig * s / 60.0 + sig * m)/60.0;
}



int double2gms_us(double x, int *g, int *m, int *s, int *us)
{
	double dm, ds;
	int sig = 1;

	if ( x < 0) {
		sig = -1;
		x *= -1.0;
	}

	*g = (int) floor(x);
	dm =  (x - (double) *g) * 60.0;
	*m = (int) floor(dm);
	ds = (dm - (double) *m) * 60.0;
	*s = (int) floor(ds);
	*us = (int) ((ds - (double) *s) * 1000000.0);
	*g *= sig;
	
	return 0;
}


int double2gms(double x, int *g, int *m, int *s)
{
	int us;

	double2gms_us(x, g, m, s, &us);
	
	return us;
}


char *double2gms_str(char *gms, double x)
{
	int g,m,s,us;

	double2gms_us(x, &g, &m, &s, &us);
	sprintf(gms,"%02d:%02d:%02d.%02d", g,m,s, 
		(int) rint( (double) us / 10000.0));

	return gms;
} 

